local addonName, addonTable = ...

addonTable.Funcs = {}

local MyFuncs = addonTable.Funcs

local Settings = {
	["bgFile"] = "Interface\\DialogFrame\\UI-DialogBox-BackGround-Dark",
	["edgeFile"] = "Interface\\Tooltips\\UI-Tooltip-Border",
	["btn_bgFile"] = "Interface\\BUTTONS\\GRADBLUE",
	["ClampedToScreen"] = false,
	["Movable"] = true,
	["Resizable"] = true,
	["pos_a"] = "CENTER",
	["pos_c"] = "CENTER",
	["pos_x"] = "-200",
	["pos_y"] = "200",
	["height"] = 150,
	["width"] = 410,
	["scale"] = 1,
	["barsize"] = 16,
	["show"] = "always",  --[Possible values: ic (in combat), ooc (out of combat), always (umm...duh?)]
}

CombatMetersDB = {
	["pos_a"] = "CENTER",
	["pos_c"] = "CENTER",
	["pos_x"] = "-200",
	["pos_y"] = "200",
	["height"] = 150,
	["width"] = 410,
	["disp"] = "dps",
}

local CombatMeters = {}
local threatTarget = "target"
local Buttons = {"Damage", "DPS", "Heals", "Interrupts", "Dispells", "Threat", "iLvl", "Reset"}
local maxScroll = 0
local numBars = 0
local pets = {}

function MyFuncs.AddComma(num)
	local retVal = num
	local i

	while true do
		retVal, i = string.gsub(retVal, "^(-?%d+)(%d%d%d)", '%1,%2')

		if i == 0 then break end
	end

	return retVal
end

function MyFuncs.ShortNumber(number)
	if not number then return end

	if number >= 1000000000000 then
		return ("%.2fT"):format(number/1000000000000)
	elseif number >= 1000000000 then
		return ("%.2fB"):format(number/1000000000)
	elseif number >= 1000000 then
		return ("%.2fM"):format(number/1000000)
	elseif number >= 99999 then
		return ("%.2fK"):format(number/1000)
	else
		return MyFuncs.AddComma(number)
	end
end

function MyFuncs.DecimalToHexColor(r, g, b, a)
	return ("|c%02x%02x%02x%02x"):format(a*255, r*255, g*255, b*255)
end

function MyFuncs.TableSum(table, key)
	local retVal = 0

	for _, n in ipairs(table) do
		if key then
			retVal = retVal + n[key]
		else
			retVal = retVal + n
		end
	end

	return retVal
end

local f = CreateFrame("Frame", "CombatMetersFrame", UIParent)

f:SetPoint("CENTER")
f:SetSize(Settings.width, Settings.height)
f:SetClampedToScreen(Settings.ClampedToScreen)
f:SetMovable(Settings.Movable)
f:SetResizable(Settings.Resizable)
f:SetMinResize(284, 66) --[[410, 50]]
f:SetUserPlaced(Settings.Movable)
f:SetBackdrop( { bgFile = Settings.bgFile, edgeFile = Settings.edgeFile, tile = true, tileSize = 32, edgeSize = 14, insets = { left = 3, right = 3, top = 3, bottom = 3 } } )
f:SetScale(Settings.scale)

local b = CreateFrame("Button", nil, CombatMetersFrame)
b:SetHeight(12)
b:SetPoint("BOTTOMLEFT", CombatMetersFrame, "BOTTOMLEFT", 4, 4)
b:SetPoint("BOTTOMRIGHT", CombatMetersFrame, "BOTTOMRIGHT", -20, 4)

local t = f:CreateFontString(f:GetName().."Author", "OVERLAY")
t:SetFont("Fonts\\ARIALN.ttf", 10)
t:SetAllPoints(b)
t:SetJustifyH("LEFT")
t:SetText("CombatMeters by |cfff58cbaIsoloedlk|r of Khaz Modan")

b:SetScript("OnEnter", function(self)
	GameTooltip:ClearLines()
	GameTooltip:SetOwner(self, "ANCHOR_BOTTOM")

	GameTooltip:AddLine(addonName.." v"..GetAddOnMetadata(addonName, "Version"))
	GameTooltip:AddLine("by |cfff58cbaIsoloedlk|r of Khaz Modan")

	GameTooltip:Show()
end)

b:SetScript("OnLeave", function(self)
	GameTooltip:Hide()
	GameTooltip:ClearLines()
end)

if Settings.Resizable then
	local g = CreateFrame("Button", "CombatMetersFrameResizeGrip", CombatMetersFrame)

	g:SetNormalTexture("Interface\\AddOns\\CombatMeters\\ResizeGrip")
	g:SetHighlightTexture("Interface\\AddOns\\CombatMeters\\ResizeGrip")
	g:SetSize(16, 16)

	g:SetScript("OnMouseDown", function(self)
		self:GetParent():StartSizing()
		self.IsMovingOrSizing = true
	end)

	g:SetScript("OnMouseUp", function(self)
		self:GetParent():StopMovingOrSizing()
		self.IsMovingOrSizing = nil
		MyFuncs.Resize()
	end)

	g:SetPoint("BOTTOMRIGHT", CombatMetersFrame, "BOTTOMRIGHT", 1, -1)

	g:SetScript("OnUpdate", function(self, elapsed)
		--self.timer = (self.timer or 0) + elapsed

		--if self.timer >= 0.2 then
			if self.IsMovingOrSizing then MyFuncs.Resize() end

		--	self.timer = 0
		--end
	end)
end

function MyFuncs.Resize()
	CombatMetersFrameSC:SetWidth(CombatMetersFrameSCParent:GetWidth())
	--CombatMetersFrameSC:SetHeight(((Settings.barsize + 2)*25)-2)

	CombatMetersDB.width = CombatMetersFrame:GetWidth()
	CombatMetersDB.height = CombatMetersFrame:GetHeight()

	if IsAddOnLoaded("EquippedItemLevelTooltip") then
		if CombatMetersFrame:GetWidth() < 459 then
			_G["CombatMetersFrame"..Buttons[7].."Button"]:Hide()
			_G["CombatMetersFrame"..Buttons[8].."Button"]:SetPoint("LEFT", _G["CombatMetersFrame"..Buttons[6].."Button"], "RIGHT", 2, 0)
		else
			_G["CombatMetersFrame"..Buttons[7].."Button"]:Show()
			_G["CombatMetersFrame"..Buttons[8].."Button"]:SetPoint("LEFT", _G["CombatMetersFrame"..Buttons[7].."Button"], "RIGHT", 2, 0)
		end
	else
		_G["CombatMetersFrame"..Buttons[7].."Button"]:Hide()
		_G["CombatMetersFrame"..Buttons[8].."Button"]:SetPoint("LEFT", _G["CombatMetersFrame"..Buttons[6].."Button"], "RIGHT", 2, 0)		
	end

	if CombatMetersFrame:GetWidth() < 410 then
		_G["CombatMetersFrame"..Buttons[4].."Button"]:Hide()
		_G["CombatMetersFrame"..Buttons[5].."Button"]:SetPoint("LEFT", _G["CombatMetersFrame"..Buttons[3].."Button"], "RIGHT", 2, 0)
	else
		_G["CombatMetersFrame"..Buttons[4].."Button"]:Show()
		_G["CombatMetersFrame"..Buttons[5].."Button"]:SetPoint("LEFT", _G["CombatMetersFrame"..Buttons[4].."Button"], "RIGHT", 2, 0)
	end

	if CombatMetersFrame:GetWidth() < 347 then
		_G["CombatMetersFrame"..Buttons[5].."Button"]:Hide()
		_G["CombatMetersFrame"..Buttons[6].."Button"]:SetPoint("LEFT", _G["CombatMetersFrame"..Buttons[3].."Button"], "RIGHT", 2, 0)
	else
		_G["CombatMetersFrame"..Buttons[5].."Button"]:Show()
		_G["CombatMetersFrame"..Buttons[6].."Button"]:SetPoint("LEFT", _G["CombatMetersFrame"..Buttons[5].."Button"], "RIGHT", 2, 0)
	end

	for i=1,numBars,1 do
--		_G["CombatMetersFrameName"..i]:SetSize(_G["CombatMetersFrameName"..i]:GetStringWidth()+20, Settings.barsize)
		_G["CombatMetersFrameDPS"..i]:SetSize(_G["CombatMetersFrameDPS"..i]:GetStringWidth()+20, Settings.barsize)
		_G["CombatMetersFrameName"..i]:SetSize(_G["CombatMetersFrameBar"..i]:GetWidth()-_G["CombatMetersFrameDPS"..i]:GetWidth(), Settings.barsize)
	end

	if CombatMetersFrameSC:GetHeight() <= CombatMetersFrameSCParent:GetHeight() then
		maxScroll = 0
	else
		maxScroll = CombatMetersFrameSC:GetHeight() - ( CombatMetersFrameSCParent:GetHeight() - ( Settings.barsize + 2 ) )
	end
end

function MyFuncs.Moving(self, button)
	if button == "RightButton" then CombatMetersFrame:StartMoving() end
end

function MyFuncs.StopMoving(self, button)
	local _
	CombatMetersDB.pos_a, _, CombatMetersDB.pos_c, CombatMetersDB.pos_x, CombatMetersDB.pos_y = CombatMetersFrame:GetPoint(1)

	CombatMetersFrame:StopMovingOrSizing()
end

local scp = CreateFrame("ScrollFrame", "CombatMetersFrameSCParent", CombatMetersFrame)
scp:SetPoint("TOPLEFT", CombatMetersFrame, "TOPLEFT", 4, -24)
scp:SetPoint("BOTTOMRIGHT", CombatMetersFrame, "BOTTOMRIGHT", -4, 20 --[[4]])

local sc = CreateFrame("Frame", "CombatMetersFrameSC", CombatMetersFrameSCParent)
sc:EnableMouse(true)
sc:EnableMouseWheel(true)

sc:SetWidth(CombatMetersFrameSCParent:GetWidth())
sc:SetHeight(((Settings.barsize + 2)*25)-2)

if Settings.Movable then
	f:SetScript("OnMouseDown", MyFuncs.Moving)
	f:SetScript("OnMouseUp", MyFuncs.StopMoving)

	scp:SetScript("OnMouseDown", MyFuncs.Moving)
	scp:SetScript("OnMouseUp", MyFuncs.StopMoving)

	sc:SetScript("OnMouseDown", Moving)
	sc:SetScript("OnMouseUp", StopMoving)
end

CombatMetersFrameSCParent:SetScrollChild(sc)

for k, v in pairs(Buttons) do
	local b = CreateFrame("Frame", "CombatMetersFrame"..v.."Button", CombatMetersFrame)
	b:ClearAllPoints()
	b:SetSize(40, (Settings.barsize + 2))
	if k == 1 then
		b:SetPoint("TOPLEFT", CombatMetersFrame, "TOPLEFT", 4, -4)
	else
		b:SetPoint("LEFT", _G["CombatMetersFrame"..Buttons[(k-1)].."Button"], "RIGHT", 2, 0)
	end

	local t = b:CreateFontString(b:GetName().."Text", "OVERLAY")
	t:SetFont("Fonts\\ARIALN.ttf", (Settings.barsize - 4), "OUTLINE")
	t:SetAllPoints(b)
	t:SetJustifyH("CENTER")
	t:SetText(v)
	b:SetWidth(t:GetStringWidth()+20)
	b:SetBackdrop( { bgFile = Settings.btn_bgFile, edgeFile = nil, tile = false, tileSize = b:GetWidth(), edgeSize = 0, insets = { left = 0, right = 0, top = 0, bottom = 0 } } )

	t:SetTextColor(1, 1, 1, 1)
	b:SetBackdropColor(1, 0.5, 0.5, 1)

	if v == "Damage" then
		b:SetScript("OnMouseDown", function(self, button)
			if button == "LeftButton" then
				if select(2, IsInInstance()) == "pvp" then
					MyFuncs.RefreshPVP("dmg")
				else
					MyFuncs.Refresh("dmg")
				end
			elseif button == "RightButton" and Settings.Movable then
				MyFuncs.Moving(self, button)
			end
		end)
		if Settings.Movable then b:SetScript("OnMouseUp", MyFuncs.StopMoving) end
	end
	if v == "DPS" then
		b:SetScript("OnMouseDown", function(self, button)
			if button == "LeftButton" then
				if select(2, IsInInstance()) == "pvp" then
					MyFuncs.RefreshPVP("dps")
				else
					MyFuncs.Refresh("dps")
				end
			elseif button == "RightButton" and Settings.Movable then
				MyFuncs.Moving(self, button)
			end
		end)
		if Settings.Movable then b:SetScript("OnMouseUp", MyFuncs.StopMoving) end
	end
	if v == "Heals" then
		b:SetScript("OnMouseDown", function(self, button)
			if button == "LeftButton" then
				if select(2, IsInInstance()) == "pvp" then
					MyFuncs.RefreshPVP("hps")
				else
					MyFuncs.Refresh("hps")
				end
			elseif button == "RightButton" and Settings.Movable then
				MyFuncs.Moving(self, button)
			end
		end)
		if Settings.Movable then b:SetScript("OnMouseUp", MyFuncs.StopMoving) end
	end
	if v == "Interrupts" then
		b:SetScript("OnMouseDown", function(self, button)
			if button == "LeftButton" then
				if select(2, IsInInstance()) ~= "pvp" then
					MyFuncs.Refresh("interrupts")
				end
			elseif button == "RightButton" and Settings.Movable then
				MyFuncs.Moving(self, button)
			end
		end)
		if Settings.Movable then b:SetScript("OnMouseUp", MyFuncs.StopMoving) end
	end
	if v == "Dispells" then
		b:SetScript("OnMouseDown", function(self, button)
			if button == "LeftButton" then
				if select(2, IsInInstance()) ~= "pvp" then
					MyFuncs.Refresh("dispells")
				end
			elseif button == "RightButton" and Settings.Movable then
				MyFuncs.Moving(self, button)
			end
		end)
		if Settings.Movable then b:SetScript("OnMouseUp", MyFuncs.StopMoving) end
	end
	if v == "Threat" then
		b:SetScript("OnMouseDown", function(self, button)
			if button == "LeftButton" then
				if select(2, IsInInstance()) ~= "pvp" then
					MyFuncs.Refresh("threat")
				end
			elseif button == "RightButton" and Settings.Movable then
				MyFuncs.Moving(self, button)
			end
		end)
		if Settings.Movable then b:SetScript("OnMouseUp", MyFuncs.StopMoving) end
	end
	if v == "iLvl" then
		b:SetScript("OnMouseDown", function(self, button)
			if button == "LeftButton" then
				if select(2, IsInInstance()) ~= "pvp" then
					MyFuncs.Refresh("ilvl")
				end
			elseif button == "RightButton" and Settings.Movable then
				MyFuncs.Moving(self, button)
			end
		end)
	end
	if v == "Reset" then
		b:SetScript("OnMouseDown", function(self, button)
			if button == "LeftButton" then
				MyFuncs.Reset()
			elseif button == "RightButton" and Settings.Movable then
				MyFuncs.Moving(self, button)
			end
		end)
		if Settings.Movable then b:SetScript("OnMouseUp", MyFuncs.StopMoving) end
	end
end

local b = CreateFrame("Frame", "CombatMetersFrameBlankButton", CombatMetersFrame)
b:ClearAllPoints()
b:SetHeight(Settings.barsize + 2)
b:SetPoint("TOPLEFT", _G["CombatMetersFrame"..Buttons[#(Buttons)].."Button"], "TOPRIGHT", 2, 0)
b:SetPoint("TOPRIGHT", CombatMetersFrame, "TOPRIGHT", -4, -2)
b:SetBackdrop( { bgFile = Settings.btn_bgFile, edgeFile = nil, tile = false, tileSize = b:GetWidth(), edgeSize = 0, insets = { left = 0, right = 0, top = 0, bottom = 0 } } )
b:SetBackdropColor(1, 0.5, 0.5, 1)
		

function MyFuncs.AddBar(i)
	if _G["CombatMetersFrameBar"..i] then return _G["CombatMetersFrameBar"..i] end

	local sb = CreateFrame("StatusBar", "CombatMetersFrameBar"..i, CombatMetersFrameSC)
	sb:SetMinMaxValues(0, 100)
	sb:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
	sb:GetStatusBarTexture():SetHorizTile(false)
	sb:SetStatusBarColor(0, 1, 0)
	sb:SetValue(0)
	sb:SetHeight(Settings.barsize)

	sb:SetPoint("TOPLEFT", CombatMetersFrameSC, "TOPLEFT", 0, (-2-(i*(Settings.barsize + 2)))+(Settings.barsize + 2))
	sb:SetPoint("TOPRIGHT", CombatMetersFrameSC, "TOPRIGHT", 0, (-2-(i*(Settings.barsize + 2)))+(Settings.barsize + 2))

	local t = sb:CreateFontString("CombatMetersFrameName"..i, "OVERLAY")
	t:SetFont("Fonts\\ARIALN.ttf", (Settings.barsize - 4), "OUTLINE")
	t:SetJustifyH("LEFT")
	t:SetPoint("LEFT", sb, "LEFT", 2, 0)

	local t = sb:CreateFontString("CombatMetersFrameDPS"..i, "OVERLAY")
	t:SetFont("Fonts\\ARIALN.ttf", (Settings.barsize - 4), "OUTLINE")
	t:SetJustifyH("RIGHT")
	t:SetPoint("RIGHT", sb, "RIGHT", -2, 0)

	--_G["CombatMetersFrameBar"..i]:SetWidth(CombatMetersFrameSC:GetWidth())
--	_G["CombatMetersFrameName"..i]:SetSize((_G["CombatMetersFrameBar"..i]:GetWidth()*0.75), Settings.barsize)
--	_G["CombatMetersFrameDPS"..i]:SetSize((_G["CombatMetersFrameBar"..i]:GetWidth()*0.25), Settings.barsize)
	_G["CombatMetersFrameName"..i]:SetSize(_G["CombatMetersFrameName"..i]:GetStringWidth(), Settings.barsize)
	_G["CombatMetersFrameDPS"..i]:SetSize(_G["CombatMetersFrameDPS"..i]:GetStringWidth(), Settings.barsize)

	CombatMetersFrameSC:SetHeight(i*(Settings.barsize + 2))
	--maxScroll = CombatMetersFrameSC:GetHeight() - CombatMetersFrameSCParent:GetHeight() -- Originally was -128 instead of CombatMetersFrameSCParent:GetHeight()
	--maxScroll = CombatMetersFrameSC:GetHeight() - ( CombatMetersFrameSCParent:GetHeight() + 20 )
	numBars = i

	MyFuncs.Resize()

	return _G["CombatMetersFrameBar"..i]
end

function MyFuncs.Reset(unitGUID)
	if unitGUID then
		CombatMeters[unitGUID] = {}
	else
		CombatMeters = nil
		CombatMeters = {}

		MyFuncs.Clear()
	end
end

function MyFuncs.AddData(srcGUID, srcName, srcClass, type, amount)
	if srcGUID == nil or srcName == nil or srcClass == nil or type == nil then return end

	if GetNumRaidMembers() >= 1 then
		for i=1,GetNumRaidMembers() do
			if UnitGUID("raid"..i) == srcGUID and UnitIsDeadOrGhost("raid"..i) then return end
		end
	elseif GetNumPartyMembers() >= 1 then
		for i=1,GetNumPartyMembers() do
			if UnitGUID("party"..i) == srcGUID and UnitIsDeadOrGhost("party"..i) then return end
		end
	end

	if UnitGUID("player") == srcGUID and UnitIsDeadOrGhost("player") then return end

	local db = CombatMeters[srcGUID] or { ["name"] = srcName, ["class"] = srcClass, ["startTime"] = GetTime(), ["endTime"] = GetTime(), ["dmg"] = 0, ["hit"] = 0, ["miss"] = 0, ["heal"] = 0, ["dispells"] = 0, ["interrupts"] = 0 }

	if type and type ~= "heal" and type ~= "miss" and type ~= "dispells" and type ~= "interrupts" then db.hit = db.hit + 1 end
	if type then db[type] = db[type] + amount end

	db.endTime = GetTime()
	if db ~= nil then CombatMeters[srcGUID] = db	end

	MyFuncs.Refresh(CombatMetersDB.disp or "dps")
end

function MyFuncs.Clear(index)
	index = (index or 0) + 1

	if index <= numBars then
		for i = index, numBars, 1 do
			_G["CombatMetersFrameBar"..i]:Hide()
		end
	end

	CombatMetersFrameSC:SetHeight(index*Settings.barsize)
end

function MyFuncs.RefreshPVP(index)
	CombatMetersFrameDPSButtonText:SetText("KBs")
	--CombatMetersFrameInterruptsButtonText:SetText("Deaths")
	--CombatMetersFrameDispellsButtonText:SetText("")

	if index == "dps" then index = "kbs" end

	local tempTbl = {}
	local sortTbl = {}
	local curData = {}
	local max = 0

	for i=1, GetNumBattlefieldScores() do
		local name, killingBlows, honorableKills, deaths, honorGained, faction, race, class, classToken, damageDone, healingDone, bgRating, ratingChange, preMatchMMR, mmrChange, talentSpec = GetBattlefieldScore(i)

		tempTbl[i] = {
			["name"] = name,
			["dmg"] = damageDone,
			["hps"] = healingDone,
			["class"] = classToken,
			["spec"] = talentSpec or "None",
			["faction"] = faction,
			["kbs"] = killingBlows,
			["hks"] = honorableKills,
			["deaths"] = deaths,
		}
		table.insert(sortTbl, i)
	end

	for k,v in pairs(Buttons) do
		if strlower(v) == index then
			_G["CombatMetersFrame"..v.."Button"]:SetBackdropColor(1, 1, 1, 1)
		else
			_G["CombatMetersFrame"..v.."Button"]:SetBackdropColor(1, 0.5, 0.5, 1)
		end
	end

	if index == "dmg" then CombatMetersFrameDamageButton:SetBackdropColor(1, 1, 1, 1) end
	if index == "hps" then CombatMetersFrameHealsButton:SetBackdropColor(1, 1, 1, 1) end
	if index == "kbs" then CombatMetersFrameDPSButton:SetBackdropColor(1, 1, 1, 1) end

	local missing

	if index == "dmg" then
		table.sort(sortTbl, function(a, b)
			return tempTbl[a].dmg > tempTbl[b].dmg
		end)

		for k, v in pairs(sortTbl) do
			curData[k] = tempTbl[v]
		end

		CombatMetersDB.disp = index
	elseif index == "kbs" then
		table.sort(sortTbl, function(a, b)
			return tempTbl[a].kbs > tempTbl[b].kbs
		end)

		for k, v in pairs(sortTbl) do
			curData[k] = tempTbl[v]
		end

		CombatMetersDB.disp = "dps"
	elseif index == "hps" then
		table.sort(sortTbl, function(a, b)
			return tempTbl[a].hps > tempTbl[b].hps
		end)

		for k, v in pairs(sortTbl) do
			curData[k] = tempTbl[v]
		end

		CombatMetersDB.disp = index
	else
		return
	end

	--for k, v in ipairs(curData) do
	--	max = max + (v[index] or 0)
	--end

	max = MyFuncs.TableSum(curData, index)

	for k, v in ipairs(curData) do
		if k <= 41 then
			local s = _G["CombatMetersFrameBar"..k] or MyFuncs.AddBar(k)
			local n = _G["CombatMetersFrameName"..k]
			local d = _G["CombatMetersFrameDPS"..k]

			if select(2, IsInInstance()) == "pvp" then
				if v.faction == 0 then
					s:SetStatusBarColor(1, 0, 0, 0.5)
				else
					s:SetStatusBarColor(0, 0, 1, 0.5)
				end
			else
				if v.faction == 0 then
					s:SetStatusBarColor(0, 1, 0, 0.5)
				else
					s:SetStatusBarColor(1, 1, 0, 0.5)
				end
			end

			if v.class and RAID_CLASS_COLORS[v.class] then
				n:SetTextColor(RAID_CLASS_COLORS[v.class].r, RAID_CLASS_COLORS[v.class].g, RAID_CLASS_COLORS[v.class].b, RAID_CLASS_COLORS[v.class].a)
			else
				n:SetTextColor(1, 1, 1, 1)
			end

			if v.spec == "Restoration" or v.spec == "Discipline" or v.spec == "Holy" then
				n:SetText(k..". "..v.name.." |cff00ff00("..v.spec..")")
			else
				n:SetText(k..". "..v.name.." ("..v.spec..")")
			end

			if index == "kbs" then
				s:SetValue(100)
				d:SetText(v.kbs.." / "..v.deaths.." / "..v.hks)
			else
				if k == 1 then missing = 100-floor((v[index]/max)*100) end

				s:SetValue(floor((v[index]/max)*100)+missing)
				d:SetText(MyFuncs.ShortNumber(v[index]).." ("..(floor((v[index]/max)*100) or 0).."%)")
			end

			--n:SetSize(n:GetStringWidth(), Settings.barsize)
			--d:SetSize(d:GetStringWidth(), Settings.barsize)
			n:SetSize(s:GetWidth()*0.75, Settings.barsize)
			d:SetSize(s:GetWidth()*0.25, Settings.barsize)

			s:Show()
		end
	end

	MyFuncs.Clear(#(sortTbl))

	MyFuncs.Resize()
end

function MyFuncs.Refresh(index)
	if ( select(2, IsInInstance()) == "pvp" or GetTime() - (CombatMetersFrame.lastUpdate or (GetTime() - 3)) < 2 ) and CombatMetersDB.disp == index then return end

	CombatMetersFrameDPSButtonText:SetText("DPS")
	--CombatMetersFrameInterruptsButtonText:SetText("Interrupts")
	--CombatMetersFrameDispellsButtonText:SetText("Dispells")

	if CombatMetersDB.disp ~= index then
		CombatMetersDB.disp = index
		MyFuncs.Clear()
	end

	local sortTbl = {}
	local curData = {}
	local max = 0

	for k,v in pairs(CombatMeters) do
		CombatMeters[k].combatTime = CombatMeters[k].endTime - CombatMeters[k].startTime
		if CombatMeters[k].combatTime < 1 then CombatMeters[k].combatTime = 2 end
		CombatMeters[k].dps = ceil(CombatMeters[k].dmg / CombatMeters[k].combatTime)
		CombatMeters[k].hps = ceil(CombatMeters[k].heal / CombatMeters[k].combatTime)
		table.insert(sortTbl, k)
	end

	for k,v in pairs(Buttons) do
		if strlower(v) == index then
			_G["CombatMetersFrame"..v.."Button"]:SetBackdropColor(1, 1, 1, 1)
		else
			_G["CombatMetersFrame"..v.."Button"]:SetBackdropColor(1, 0.5, 0.5, 1)
		end
	end

	if index == "dmg" then CombatMetersFrameDamageButton:SetBackdropColor(1, 1, 1, 1) end
	if index == "hps" then CombatMetersFrameHealsButton:SetBackdropColor(1, 1, 1, 1) end

	local dps1, dps2, missing

	if index == "dmg" then
		table.sort(sortTbl, function(a, b)
			return CombatMeters[a].dmg > CombatMeters[b].dmg
		end)

		for k, v in pairs(sortTbl) do
			curData[k] = CombatMeters[v]
			curData[k].unitGUID = v
		end
	elseif index == "dps" then
		table.sort(sortTbl, function(a, b)
			local time1 = CombatMeters[a].endTime-CombatMeters[a].startTime
			if time1 < 1 then time1 = 2 end

			dps1 = (CombatMeters[a].dmg / time1)

			local time2 = CombatMeters[b].endTime-CombatMeters[b].startTime
			if time2 < 1 then time2 = 2 end

			dps2 = (CombatMeters[b].dmg / time2)

			return dps1 > dps2
		end)

		for k, v in pairs(sortTbl) do
			curData[k] = CombatMeters[v]
			curData[k].unitGUID = v
		end
	elseif index == "hps" then
		table.sort(sortTbl, function(a, b)
			local time1 = CombatMeters[a].endTime-CombatMeters[a].startTime
			if time1 < 1 then time1 = 2 end

			dps1 = (CombatMeters[a].heal / time1)

			local time2 = CombatMeters[b].endTime-CombatMeters[b].startTime
			if time2 < 1 then time2 = 2 end

			dps2 = (CombatMeters[b].heal / time2)

			return dps1 > dps2
		end)

		for k, v in pairs(sortTbl) do
			curData[k] = CombatMeters[v]
			curData[k].unitGUID = v
		end
	elseif index == "dispells" then
		table.sort(sortTbl, function(a, b)
			return CombatMeters[a].dispells > CombatMeters[b].dispells
		end)

		for k, v in pairs(sortTbl) do
			curData[k] = CombatMeters[v]
			curData[k].unitGUID = v
		end
	elseif index == "interrupts" then
		table.sort(sortTbl, function(a, b)
			return CombatMeters[a].interrupts > CombatMeters[b].interrupts
		end)

		for k, v in pairs(sortTbl) do
			curData[k] = CombatMeters[v]
			curData[k].unitGUID = v
		end
	elseif index == "threat" then
		if not threatTarget or UnitIsFriend("player", (threatTarget or "target")) then return end

		local threatTbl = {}
		local sortTbl = {}

		if GetNumRaidMembers() > 0 then
			for i=1,GetNumRaidMembers(),1 do
				local isTanking, status, scaledPercent, rawPercent, threatValue = UnitDetailedThreatSituation("raid"..i, threatTarget)

				table.insert(threatTbl, {["isTanking"] = isTanking or 0, ["status"] = status or 0, ["scaledPercent"] = scaledPercent or 0, ["rawPercent"] = rawPercent or 0, ["threatValue"] = threatValue or 0, ["unit"] = "raid"..i, ["name"] = UnitName("raid"..i) })

				if UnitExists("raid"..i.."pet") then
					local isTanking, status, scaledPercent, rawPercent, threatValue = UnitDetailedThreatSituation("raid"..i.."pet", threatTarget)

					table.insert(threatTbl, {["isTanking"] = isTanking or 0, ["status"] = status or 0, ["scaledPercent"] = scaledPercent or 0, ["rawPercent"] = rawPercent or 0, ["threatValue"] = threatValue, ["unit"] = "raid"..i.."pet", ["name"] = UnitName("raid"..i.."pet") })
				end
			end
		elseif GetNumPartyMembers() > 0 then
			for i=1,GetNumPartyMembers(),1 do
				local isTanking, status, scaledPercent, rawPercent, threatValue = UnitDetailedThreatSituation("party"..i, threatTarget)

				table.insert(threatTbl, {["isTanking"] = isTanking or 0, ["status"] = status or 0, ["scaledPercent"] = scaledPercent or 0, ["rawPercent"] = rawPercent or 0, ["threatValue"] = threatValue or 0, ["unit"] = "party"..i, ["name"] = UnitName("party"..i) })
	
				if UnitExists("party"..i.."pet") then
					local isTanking, status, scaledPercent, rawPercent, threatValue = UnitDetailedThreatSituation("party"..i.."pet", threatTarget)

					table.insert(threatTbl, {["isTanking"] = isTanking or 0, ["status"] = status or 0, ["scaledPercent"] = scaledPercent or 0, ["rawPercent"] = rawPercent or 0, ["threatValue"] = threatValue or 0, ["unit"] = "party"..i.."pet", ["name"] = UnitName("party"..i.."pet") })
				end
			end

			local isTanking, status, scaledPercent, rawPercent, threatValue = UnitDetailedThreatSituation("player", threatTarget)

			table.insert(threatTbl, {["isTanking"] = isTanking or 0, ["status"] = status or 0, ["scaledPercent"] = scaledPercent or 0, ["rawPercent"] = rawPercent or 0, ["threatValue"] = threatValue or 0, ["unit"] = "player", ["name"] = UnitName("player") })

			if UnitExists("playerpet") then
				local isTanking, status, scaledPercent, rawPercent, threatValue = UnitDetailedThreatSituation("playerpet", threatTarget)

				table.insert(threatTbl, {["isTanking"] = isTanking or 0, ["status"] = status or 0, ["scaledPercent"] = scaledPercent or 0, ["rawPercent"] = rawPercent or 0, ["threatValue"] = threatValue or 0, ["unit"] = "playerpet", ["name"] = UnitName("playerpet") })
			end
		else
			local isTanking, status, scaledPercent, rawPercent, threatValue = UnitDetailedThreatSituation("player", threatTarget)

			table.insert(threatTbl, {["isTanking"] = isTanking or 0, ["status"] = status or 0, ["scaledPercent"] = scaledPercent or 0, ["rawPercent"] = rawPercent or 0, ["threatValue"] = threatValue or 0, ["unit"] = "player", ["name"] = UnitName("player") })

			if UnitExists("playerpet") then
				local isTanking, status, scaledPercent, rawPercent, threatValue = UnitDetailedThreatSituation("playerpet", threatTarget)
	
				table.insert(threatTbl, {["isTanking"] = isTanking or 0, ["status"] = status or 0, ["scaledPercent"] = scaledPercent or 0, ["rawPercent"] = rawPercent or 0, ["threatValue"] = threatValue or 0, ["unit"] = "playerpet", ["name"] = UnitName("playerpet") })
			end
		end

		for k,v in pairs(threatTbl) do table.insert(sortTbl, k) end

		table.sort(sortTbl, function(a,b) return threatTbl[a].rawPercent > threatTbl[b].rawPercent end)

		for k,v in ipairs(sortTbl) do
			if k <= 41 then
				local s = _G["CombatMetersFrameBar"..k] or MyFuncs.AddBar(k)
				local n = _G["CombatMetersFrameName"..k]
				local d = _G["CombatMetersFrameDPS"..k]

				n:SetText(k..". "..threatTbl[v].name)
				d:SetText((MyFuncs.ShortNumber(threatTbl[v].threatValue) or 0).." ("..string.format("%.0f", threatTbl[v].scaledPercent).."%)")
				s:SetValue(threatTbl[v].scaledPercent)

				if threatTbl[v].scaledPercent > 0 then
					if (threatTbl[v].unit):find("pet") then
						s:SetStatusBarColor(0, 1, 0, 1)
					else
						s:SetStatusBarColor(RAID_CLASS_COLORS[select(2, UnitClass(threatTbl[v].unit))].r, RAID_CLASS_COLORS[select(2, UnitClass(threatTbl[v].unit))].g, RAID_CLASS_COLORS[select(2, UnitClass(threatTbl[v].unit))].b, RAID_CLASS_COLORS[select(2, UnitClass(threatTbl[v].unit))].a)
					end

					--n:SetSize(n:GetStringWidth(), Settings.barsize)
					--d:SetSize(d:GetStringWidth(), Settings.barsize)
					n:SetSize(s:GetWidth()*0.75, Settings.barsize)
					d:SetSize(s:GetWidth()*0.25, Settings.barsize)

					s:Show()
				else
					s:Hide()
				end
			end
		end

		return
	elseif index == "ilvl" then
		local sortTbl = {}

		if GetNumRaidMembers() >= 1 then
			for i=1,GetNumRaidMembers() do
				tinsert(sortTbl, UnitGUID("raid"..i))
				if not UnitItemLevelDB[UnitGUID("raid"..i)] then
					UnitItemLevelDB[UnitGUID("raid"..i)] = { ["count"] = 0, ["equipped"] = 0 }
				end
			end
		elseif GetNumPartyMembers() >= 1 then
			for i=1,GetNumPartyMembers() do
				tinsert(sortTbl, UnitGUID("party"..i))
				if not UnitItemLevelDB[UnitGUID("party"..i)] then
					UnitItemLevelDB[UnitGUID("party"..i)] = { ["count"] = 0, ["equipped"] = 0 }
				end
			end

			tinsert(sortTbl, UnitGUID("player"))
		else
			tinsert(sortTbl, UnitGUID("player"))
		end

		UnitItemLevelDB[UnitGUID("player")] = { ["count"] = 2, ["equipped"] = select(2, GetAverageItemLevel()) }

		table.sort(sortTbl, function(a, b)
			return (UnitItemLevelDB[a].equipped or 0) > (UnitItemLevelDB[b].equipped or 0)
		end)

		for k, v in pairs(sortTbl) do
			local _, classFilename, _, _, _, name, realm = GetPlayerInfoByGUID(v)

			if not classFilename or not name then return end

			curData[k] = UnitItemLevelDB[v]
			curData[k].unitGUID = v
			curData[k].name = name
			curData[k].class = classFilename
		end

		for k, v in ipairs(curData) do
			if k <= 41 then
				local s = _G["CombatMetersFrameBar"..k] or MyFuncs.AddBar(k)
				local n = _G["CombatMetersFrameName"..k]
				local d = _G["CombatMetersFrameDPS"..k]

				s:SetStatusBarColor(RAID_CLASS_COLORS[v.class].r, RAID_CLASS_COLORS[v.class].g, RAID_CLASS_COLORS[v.class].b, RAID_CLASS_COLORS[v.class].a)
				n:SetTextColor(1, 1, 1, 1)
	
				n:SetText(k..". "..v.name)

				s:SetValue(100)
				--d:SetText(MyFuncs.ShortNumber(floor(v.equipped or 0)))

				if (v.equipped or 0) > 0 then
					--n:SetSize(s:GetWidth()*0.75, Settings.barsize)
					--d:SetSize(s:GetWidth()*0.25, Settings.barsize)

					--s:Show()

					d:SetText(MyFuncs.ShortNumber(floor(v.equipped or 0)))
				else
					d:SetText("Not Scanned")
					--s:Hide()
				end

				s:Show()
			end
		end

		CombatMetersFrame.lastUpdate = GetTime()
		MyFuncs.Clear(#(sortTbl))
		MyFuncs.Resize()

		return
	end

	for k, v in ipairs(curData) do
		max = max + (v[index] or 0)
	end

	for k, v in ipairs(curData) do
		if k <= 41 then
			local s = _G["CombatMetersFrameBar"..k] or MyFuncs.AddBar(k)
			local n = _G["CombatMetersFrameName"..k]
			local d = _G["CombatMetersFrameDPS"..k]

			s:SetStatusBarColor(RAID_CLASS_COLORS[v.class].r, RAID_CLASS_COLORS[v.class].g, RAID_CLASS_COLORS[v.class].b, RAID_CLASS_COLORS[v.class].a)
			n:SetTextColor(1, 1, 1, 1)

			n:SetText(k..". "..v.name)
			if k == 1 then missing = 100-floor((v[index]/max)*100) end

			if max == 0 then
				s:SetValue(100)
				d:SetText(MyFuncs.ShortNumber(v[index]).." (0%)")
			else
				s:SetValue(floor((v[index] / max)*100)+missing)
				d:SetText(MyFuncs.ShortNumber(v[index]).." ("..floor((v[index] / max)*100).."%)")
			end
			if v[index] > 0 then
				--n:SetSize(n:GetStringWidth(), Settings.barsize)
				--d:SetSize(d:GetStringWidth(), Settings.barsize)
				n:SetSize(s:GetWidth()*0.75, Settings.barsize)
				d:SetSize(s:GetWidth()*0.25, Settings.barsize)

				s:Show()
			else
				s:Hide()
			end
		end
	end

	CombatMetersFrame.lastUpdate = GetTime()

	MyFuncs.Clear(#(sortTbl))

	MyFuncs.Resize()
end

local function AssociatePets()
	if GetNumRaidMembers() >= 1 then
		for i=1,40 do
			if UnitExists("raid"..i.."pet") then
				pets[UnitGUID("raid"..i.."pet")] = UnitGUID("raid"..i)
			end
		end
	end

	if GetNumPartyMembers() >= 1 then
		for i=1,5 do
			if UnitExists("party"..i.."pet") then
				pets[UnitGUID("party"..i.."pet")] = UnitGUID("party"..i)
			end
		end
	end
end

local function OnEvent(self, event, ...)
	if event == "ADDON_LOADED" then
		if addonName == ... then
			DEFAULT_CHAT_FRAME:AddMessage("|cffff8000CombatMeters|r loaded. |cff00ff00/cm|r or |cff00ff00/combatmeters|r for list of options.")
			--CombatMetersConfig:Init()
		end
	elseif event == "VARIABLES_LOADED" then
		self:ClearAllPoints()

		if Settings.Movable == true then
			self:SetPoint(CombatMetersDB.pos_a, UIParent, CombatMetersDB.pos_c, CombatMetersDB.pos_x, CombatMetersDB.pos_y)
		else
			self:SetPoint(Settings.pos_a, UIParent, Settings.pos_c, Settings.pos_x, Settings.pos_y)
		end

		if Settings.show == "ooc" or Settings.show == "always" then
			self:Show()
		else
			self:Hide()
		end

		if Settings.width == 410 or Settings.height == 150 then
			self:SetSize(CombatMetersDB.width or Settings.width, CombatMetersDB.height or Settings.height)
		end

		if CombatMetersDB.scale then
			CombatMetersFrame:SetScale(CombatMetersDB.scale or 1)
		else
			CombatMetersFrame:SetScale(Settings.scale or 1)
		end

		MyFuncs.Resize()

		self:UnregisterEvent("VARIABLES_LOADED")
	elseif event == "RAID_ROSTER_UPDATE" or event == "PARTY_MEMBERS_CHANGED" then
		AssociatePets()
	elseif event == "PLAYER_REGEN_ENABLED" then
		if Settings.show == "ooc" then
			self:Show()
		elseif Settings.show == "ic" then
			self:Hide()
		end

		self.lastCombatEnd = GetTime()
	elseif event == "PLAYER_REGEN_DISABLED" then
		if Settings.show == "ooc" then
			self:Hide()
		elseif Settings.show == "ic" then
			self:Show()
		end

		if GetTime() - self.lastCombatEnd >= 2 then
			CombatMeters = {}
		end
	elseif event == "UNIT_THREAT_LIST_UPDATE" then
		if select(2, IsInInstance()) ~= "pvp" and select(2, IsInInstance()) ~= "arena" then
			local tgt = ... or "target"

			if UnitExists(tgt) and not UnitIsFriend("player", tgt) then
				threatTarget = tgt
			else
				threatTarget = "target"
			end
		end
	elseif event == "PLAYER_ENTERING_WORLD" then
		if select(2, IsInInstance()) == "pvp" then
			MyFuncs.RefreshPVP(CombatMetersDB.disp or "dps")
		else
			MyFuncs.Refresh(CombatMetersDB.disp or "dps")
		end
	elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
		local timestamp, event, hideCaster, srcGUID, srcName, srcFlags, _, dstGUID, dstName, dstFlags = ...

		if srcName == "" or hideCaster then return end

		if not ( bit.band(srcFlags, COMBATLOG_OBJECT_AFFILIATION_RAID) > 0 or bit.band(srcFlags, COMBATLOG_OBJECT_AFFILIATION_PARTY) > 0 or bit.band(srcFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 ) then return end

		if bit.band(srcFlags, COMBATLOG_OBJECT_REACTION_FRIENDLY) > 0 and bit.band(srcFlags, COMBATLOG_OBJECT_CONTROL_PLAYER) > 0 and srcName ~= nil and srcGUID ~= nil then
			local srcClass

			if bit.band(srcFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				srcClass = select(2, GetPlayerInfoByGUID(srcGUID))
			else
				if bit.band(srcFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then
					srcGUID = UnitGUID("player")
					srcName = UnitName("player")
					srcClass = select(2, UnitClass("player"))
				else
					AssociatePets()

					if srcGUID ~= "" and srcGUID ~= nil then
						if srcGUID ~= "" and srcGUID ~= nil then
							srcGUID = pets[srcGUID]
							if srcGUID ~= "" and srcGUID ~= nil then
								_, srcClass, _, _, _, srcName = GetPlayerInfoByGUID(srcGUID)
							end
						else
							srcClass = "pet"
						end
					else
						srcClass = "pet"
					end
				end
			end

			if string.find(event, "_INTERRUPT") then
				MyFuncs.AddData(srcGUID, srcName, srcClass, "interrupts", 1)
			elseif string.find(event, "_DISPELL") then
				MyFuncs.AddData(srcGUID, srcName, srcClass, "dispells", 1)
			elseif string.find(event, "_HEAL") then
				local amount, _, absorbed = select(15, ...)

				MyFuncs.AddData(srcGUID, srcName, srcClass, "heal", ((amount or 0) + (absorbed or 0)) )
			elseif string.find(event, "_MISS") then
				MyFuncs.AddData(srcGUID, srcName, srcClass, "miss", 1)
			elseif string.find(event, "_DAMAGE") or string.find(event, "_EXTRA_ATTACKS") then
				local amount

				if strsub(event, 1, 5) == "SWING" then
					amount = select(12, ...)
				else
					amount = select(15, ...)
				end

				MyFuncs.AddData(srcGUID, srcName, srcClass, "dmg", amount)
			elseif string.find(event, "_DRAIN") or string.find(event, "_LEECH") then
				local spellID, spellName, spellSchool, amount, powerType = select(12, ...)

				if powerType == -2 then
					MyFuncs.AddData(srcGUID, srcName, srcClass, "dmg", amount)
				else
					MyFuncs.AddData(srcGUID, srcName, srcClass, nil, amount)
				end
			elseif string.find(event, "_SUMMON") then
				if dstGUID ~= nil and dstGUID ~= "" and srcGUID ~= nil and srcGUID ~= "" then
					pets[dstGUID] = srcGUID
				end
			end
		end
	end
end

f:RegisterEvent("VARIABLES_LOADED")
f:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
f:RegisterEvent("PLAYER_REGEN_ENABLED")
f:RegisterEvent("PLAYER_REGEN_DISABLED")
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("PARTY_MEMBERS_CHANGED")
f:RegisterEvent("RAID_ROSTER_UPDATE")
f:RegisterEvent("UNIT_THREAT_LIST_UPDATE")
f:RegisterEvent("ADDON_LOADED")

f:SetScript("OnEvent", OnEvent)

f:SetScript("OnUpdate", function(self, elapsed)
	self.timer = (self.timer or 0) + elapsed

	if self.timer >= 2 then
		if select(2, IsInInstance()) == "pvp" then MyFuncs.RefreshPVP(CombatMetersDB.disp or "dps") end

		if CombatMetersDB.disp == "ilvl" and IsAddOnLoaded("EquippedItemLevelTooltip") then
			MyFuncs.Refresh(CombatMetersDB.disp or "ilvl")
		end

		self.timer = 0
	end
end)

f:SetScript("OnMouseWheel", function(self, delta)
	local scp = CombatMetersFrameSCParent

	if delta > 0 then
		if scp:GetVerticalScroll() > 20 then
			scp:SetVerticalScroll(scp:GetVerticalScroll()-20)
		else
			scp:SetVerticalScroll(0)
		end
	else
		if scp:GetVerticalScroll() < maxScroll then
			scp:SetVerticalScroll(scp:GetVerticalScroll()+20)
		else
			scp:SetVerticalScroll(maxScroll)
		end
	end
end)

function MyFuncs.SlashCmd(...)
	local cmd, opt = (...):match("(.+) (.+)")

	if ... == "show" then
		CombatMetersFrame:Show()
	elseif ... == "hide" then
		CombatMetersFrame:Hide()
	elseif ... == "toggle" then
		if CombatMetersFrame:IsVisible() then
			CombatMetersFrame:Hide()
		else
			CombatMetersFrame:Show()
		end
	elseif cmd == "scale" then
		opt = tonumber(opt) or 1

		if type(opt) == "number" then
			if opt >= 0.1 and opt <= 2 then
				CombatMetersDB.scale = opt
				CombatMetersFrame:SetScale(opt)
			else
				DEFAULT_CHAT_FRAME:AddMessage("|cffff8000CombatMeters:|r scale must be 0.1 <=> 2.")
			end
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cffff8000CombatMeters:|r unknown value for scale.")
		end
	elseif (...):sub(1, 3) == "dmg" or (...):sub(1, 3) == "dam" then
		if select(2, IsInInstance()) == "pvp" then
			MyFuncs.RefreshPVP("dmg")
		else
			MyFuncs.Refresh("dmg")
		end
	elseif (...):sub(1, 3) == "dps" or (...):sub(1, 3) == "kbs" then
		if select(2, IsInInstance()) == "pvp" then
			MyFuncs.RefreshPVP("kbs")
		else
			MyFuncs.Refresh("dps")
		end
	elseif (...):sub(1, 3) == "hea" or (...):sub(1, 3) == "hps" then
		if select(2, IsInInstance()) == "pvp" then
			MyFuncs.RefreshPVP("hps")
		else
			MyFuncs.Refresh("hps")
		end
	elseif (...):sub(1, 3) == "int" then
		if select(2, IsInInstance()) ~= "pvp" then
			MyFuncs.Refresh("interrupts")
		end
	elseif (...):sub(1, 3) == "dis" then
		if select(2, IsInInstance()) ~= "pvp" then
			MyFuncs.Refresh("dispells")
		end
	elseif (...):sub(1, 3) == "thr" then
		if select(2, IsInInstance()) ~= "pvp" then
			MyFuncs.Refresh("threat")
		end
	elseif (...):sub(1, 3) == "ilv" or (...):sub(1, 3) == "ile" then
		if select(2, IsInInstance()) == "pvp" then
			MyFuncs.RefreshPVP("ilvl")
		else
			MyFuncs.Refresh("ilvl")
		end
	elseif ... == "config" then
		--CombatMetersConfig:OpenConfig()
	elseif ... == "reset" then
		CombatMetersFrame:ClearAllPoints()
		CombatMetersFrame:SetPoint("CENTER", UIParent, "CENTER", -200, 200)
	elseif ... == "maxScroll" then
		DEFAULT_CHAT_FRAME:AddMessage("|cffff8000CombatMeters|r: maxScroll = "..maxScroll)
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cffff8000CombatMeters: |cff00ff00/cm|cffffffff or |cff00ff00/combatmeters")
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff00show|r - Show main window.")
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff00hide|r - Hide main window.")
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff00toggle|r - Toggle show/hide main window.")
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff00scale <#>|r - Set scale from 0.1 to 2.")
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff00reset|r - Reset main window position.")
	end
end

SLASH_COMBATMETERS1 = "/combatmeters"
SLASH_COMBATMETERS2 = "/cm"
SlashCmdList["COMBATMETERS"] = MyFuncs.SlashCmd
